# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: models/core/core.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='models/core/core.proto',
  package='core',
  syntax='proto3',
  serialized_options=b'Z\037go.jetpack.io/axiom/models/core',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x16models/core/core.proto\x12\x04\x63ore\x1a\x1fgoogle/protobuf/timestamp.proto\"v\n\x08NodeMeta\x12\n\n\x02id\x18\x01 \x01(\t\x12.\n\ncreated_at\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12.\n\nupdated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\x1d\n\x0bPointerMeta\x12\x0e\n\x06loaded\x18\x01 \x01(\x08\"\x0b\n\tNonDBMetaB!Z\x1fgo.jetpack.io/axiom/models/coreb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])




_NODEMETA = _descriptor.Descriptor(
  name='NodeMeta',
  full_name='core.NodeMeta',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='core.NodeMeta.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='created_at', full_name='core.NodeMeta.created_at', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='updated_at', full_name='core.NodeMeta.updated_at', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=65,
  serialized_end=183,
)


_POINTERMETA = _descriptor.Descriptor(
  name='PointerMeta',
  full_name='core.PointerMeta',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='loaded', full_name='core.PointerMeta.loaded', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=185,
  serialized_end=214,
)


_NONDBMETA = _descriptor.Descriptor(
  name='NonDBMeta',
  full_name='core.NonDBMeta',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=216,
  serialized_end=227,
)

_NODEMETA.fields_by_name['created_at'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_NODEMETA.fields_by_name['updated_at'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
DESCRIPTOR.message_types_by_name['NodeMeta'] = _NODEMETA
DESCRIPTOR.message_types_by_name['PointerMeta'] = _POINTERMETA
DESCRIPTOR.message_types_by_name['NonDBMeta'] = _NONDBMETA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

NodeMeta = _reflection.GeneratedProtocolMessageType('NodeMeta', (_message.Message,), {
  'DESCRIPTOR' : _NODEMETA,
  '__module__' : 'models.core.core_pb2'
  # @@protoc_insertion_point(class_scope:core.NodeMeta)
  })
_sym_db.RegisterMessage(NodeMeta)

PointerMeta = _reflection.GeneratedProtocolMessageType('PointerMeta', (_message.Message,), {
  'DESCRIPTOR' : _POINTERMETA,
  '__module__' : 'models.core.core_pb2'
  # @@protoc_insertion_point(class_scope:core.PointerMeta)
  })
_sym_db.RegisterMessage(PointerMeta)

NonDBMeta = _reflection.GeneratedProtocolMessageType('NonDBMeta', (_message.Message,), {
  'DESCRIPTOR' : _NONDBMETA,
  '__module__' : 'models.core.core_pb2'
  # @@protoc_insertion_point(class_scope:core.NonDBMeta)
  })
_sym_db.RegisterMessage(NonDBMeta)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
